(require [kodhy.macros [rmap qw getl geti $ λ wc ss cbind replicate]])

(import
  [math [log]]
  [numpy :as np]
  [pandas :as pd]
  [statsmodels.stats.proportion [proportion-confint]]
  [kodhy.util [T F rd odds-ratio valcounts pretty-json-to-pd]])

(setv tz-received "Africa/Johannesburg")

(setv obs (pretty-json-to-pd "../data.json"))
(setv ($ obs received_time)
  (.dt.tz-convert :tz tz-received
  (.dt.tz-localize :tz "UTC"
  (pd.to-datetime ($ obs received_time) :unit "s"))))

(setv sb
  ; sb will be like obs, but with only the first observation
  ; for each subject, and only including observations up to
  ; and not including February 2017.
  (.apply :func (λ (geti it 0))
  (.groupby :by "s"
  (.sort-values :by "received_time"
  (ss obs (< $received_time "2017-02-01"))))))

(setv dr (wc sb (cbind
  :booze_rdt $drug_alcohol_rdt
  :booze_sr $drug_alcohol_last3days
  :weed_rdt $drug_cannabis_rdt
  :weed_sr $drug_cannabis_last2days
  :meth_rdt $drug_methamphetamine_rdt
  :meth_sr $drug_methamphetamine_last2days)))

(setv drug-validity (pd.concat (rmap [drug (qw booze weed meth)]
  (setv d (cbind :drug drug (.reset-index (.apply
    (.groupby dr [(+ drug "_rdt") (+ drug "_sr")])
    len))))
  (setv d.columns (qw drug rdt sr n))
  d)))
(setv ($ drug-validity drug) (.reorder-categories
  (. (.astype ($ drug-validity drug) "category") cat)
  (qw booze weed meth)))

(defn drug-cm-stat [selector grouper]
  (setv x drug-validity)
  (unless (is selector T)
    (setv x (eval `(ss x ~selector))))
  (setv x (.sum (.groupby x ["drug" (eval `(wc x ~grouper))])))
  (setv x ($ x n))
  (setv x (/ (getl x : T) (.sum x :level "drug")))
  (.round x 2))
